@@ -1,5 +1,4 @@
-#ifndef I3_XCB_H
-#define I3_XCB_H
+#pragma once
 
 /* from X11/keysymdef.h */
 #define XCB_NUM_LOCK                    0xff7f
@@ -7,5 +6,3 @@
 #define xmacro(atom) xcb_atom_t A_ ## atom;
 #include "atoms.xmacro"
 #undef xmacro
-
-#endif
