@@ -76,20 +76,17 @@ static int reply_boolean_cb(void *params, int val) {
     return 1;
 }
 
-#if YAJL_MAJOR >= 2
 static int reply_string_cb(void *params, const unsigned char *val, size_t len) {
-#else
-static int reply_string_cb(void *params, const unsigned char *val, unsigned int len) {
-#endif
     char *str = scalloc(len + 1);
-    strncpy(str, (const char*)val, len);
+    strncpy(str, (const char *)val, len);
     if (strcmp(last_key, "error") == 0)
         last_reply.error = str;
     else if (strcmp(last_key, "input") == 0)
         last_reply.input = str;
     else if (strcmp(last_key, "errorposition") == 0)
         last_reply.errorposition = str;
-    else free(str);
+    else
+        free(str);
     return 1;
 }
 
@@ -106,36 +103,25 @@ static int reply_end_map_cb(void *params) {
     return 1;
 }
 
-
-#if YAJL_MAJOR >= 2
 static int reply_map_key_cb(void *params, const unsigned char *keyVal, size_t keyLen) {
-#else
-static int reply_map_key_cb(void *params, const unsigned char *keyVal, unsigned keyLen) {
-#endif
     free(last_key);
     last_key = scalloc(keyLen + 1);
-    strncpy(last_key, (const char*)keyVal, keyLen);
+    strncpy(last_key, (const char *)keyVal, keyLen);
     return 1;
 }
 
-yajl_callbacks reply_callbacks = {
-    NULL,
-    &reply_boolean_cb,
-    NULL,
-    NULL,
-    NULL,
-    &reply_string_cb,
-    &reply_start_map_cb,
-    &reply_map_key_cb,
-    &reply_end_map_cb,
-    NULL,
-    NULL
+static yajl_callbacks reply_callbacks = {
+    .yajl_boolean = reply_boolean_cb,
+    .yajl_string = reply_string_cb,
+    .yajl_start_map = reply_start_map_cb,
+    .yajl_map_key = reply_map_key_cb,
+    .yajl_end_map = reply_end_map_cb,
 };
 
 int main(int argc, char *argv[]) {
     socket_path = getenv("I3SOCK");
     int o, option_index = 0;
-    int message_type = I3_IPC_MESSAGE_TYPE_COMMAND;
+    uint32_t message_type = I3_IPC_MESSAGE_TYPE_COMMAND;
     char *payload = NULL;
     bool quiet = false;
 
@@ -145,8 +131,7 @@ int main(int argc, char *argv[]) {
         {"version", no_argument, 0, 'v'},
         {"quiet", no_argument, 0, 'q'},
         {"help", no_argument, 0, 'h'},
-        {0, 0, 0, 0}
-    };
+        {0, 0, 0, 0}};
 
     char *options_string = "s:t:vhq";
 
@@ -221,10 +206,10 @@ int main(int argc, char *argv[]) {
     memset(&addr, 0, sizeof(struct sockaddr_un));
     addr.sun_family = AF_LOCAL;
     strncpy(addr.sun_path, socket_path, sizeof(addr.sun_path) - 1);
-    if (connect(sockfd, (const struct sockaddr*)&addr, sizeof(struct sockaddr_un)) < 0)
+    if (connect(sockfd, (const struct sockaddr *)&addr, sizeof(struct sockaddr_un)) < 0)
         err(EXIT_FAILURE, "Could not connect to i3 on socket \"%s\"", socket_path);
 
-    if (ipc_send_message(sockfd, strlen(payload), message_type, (uint8_t*)payload) == -1)
+    if (ipc_send_message(sockfd, strlen(payload), message_type, (uint8_t *)payload) == -1)
         err(EXIT_FAILURE, "IPC: write()");
 
     if (quiet)
@@ -245,21 +230,12 @@ int main(int argc, char *argv[]) {
      * If not, nicely format the error message. */
     if (reply_type == I3_IPC_MESSAGE_TYPE_COMMAND) {
         yajl_handle handle;
-#if YAJL_MAJOR < 2
-        yajl_parser_config parse_conf = { 0, 0 };
-
-        handle = yajl_alloc(&reply_callbacks, &parse_conf, NULL, NULL);
-#else
         handle = yajl_alloc(&reply_callbacks, NULL, NULL);
-#endif
-        yajl_status state = yajl_parse(handle, (const unsigned char*)reply, reply_length);
+        yajl_status state = yajl_parse(handle, (const unsigned char *)reply, reply_length);
         switch (state) {
             case yajl_status_ok:
                 break;
             case yajl_status_client_canceled:
-#if YAJL_MAJOR < 2
-            case yajl_status_insufficient_data:
-#endif
             case yajl_status_error:
                 errx(EXIT_FAILURE, "IPC: Could not parse JSON reply.");
         }
