@@ -7,10 +7,7 @@
  * key_press.c: key press handler
  *
  */
-#ifndef I3_KEY_PRESS_H
-#define I3_KEY_PRESS_H
-
-extern pid_t command_error_nagbar_pid;
+#pragma once
 
 /**
  * There was a key press. We compare this key code with our bindings table and pass
@@ -30,5 +27,3 @@ void handle_key_press(xcb_key_press_event_t *event);
  *
  */
 void kill_commanderror_nagbar(bool wait_for_it);
-
-#endif
