@@ -11,8 +11,7 @@
  * match_matches_window() to find the windows affected by this command.
  *
  */
-#ifndef I3_MATCH_H
-#define I3_MATCH_H
+#pragma once
 
 /*
  * Initializes the Match data structure. This function is necessary because the
@@ -46,5 +45,3 @@ bool match_matches_window(Match *match, i3Window *window);
  *
  */
 void match_free(Match *match);
-
-#endif
