@@ -7,11 +7,8 @@
  * resize.c: Interactive resizing.
  *
  */
-#ifndef I3_RESIZE_H
-#define I3_RESIZE_H
+#pragma once
 
 bool resize_find_tiling_participants(Con **current, Con **other, direction_t direction);
 
 int resize_graphical_handler(Con *first, Con *second, orientation_t orientation, const xcb_button_press_event_t *event);
-
-#endif
