@@ -37,7 +37,7 @@ struct focus_mapping {
 };
 
 static TAILQ_HEAD(focus_mappings_head, focus_mapping) focus_mappings =
-  TAILQ_HEAD_INITIALIZER(focus_mappings);
+    TAILQ_HEAD_INITIALIZER(focus_mappings);
 
 static int json_start_map(void *ctx) {
     LOG("start of map, last_key = %s\n", last_key);
@@ -67,6 +67,23 @@ static int json_start_map(void *ctx) {
 static int json_end_map(void *ctx) {
     LOG("end of map\n");
     if (!parsing_swallows && !parsing_rect && !parsing_window_rect && !parsing_geometry) {
+        /* Set a few default values to simplify manually crafted layout files. */
+        if (json_node->layout == L_DEFAULT) {
+            DLOG("Setting layout = L_SPLITH\n");
+            json_node->layout = L_SPLITH;
+        }
+
+        /* Sanity check: swallow criteria don’t make any sense on a split
+         * container. */
+        if (con_is_split(json_node) > 0 && !TAILQ_EMPTY(&(json_node->swallow_head))) {
+            DLOG("sanity check: removing swallows specification from split container\n");
+            while (!TAILQ_EMPTY(&(json_node->swallow_head))) {
+                Match *match = TAILQ_FIRST(&(json_node->swallow_head));
+                TAILQ_REMOVE(&(json_node->swallow_head), match, matches);
+                match_free(match);
+            }
+        }
+
         LOG("attaching\n");
         con_attach(json_node, json_node->parent, true);
         LOG("Creating window\n");
@@ -84,14 +101,19 @@ static int json_end_map(void *ctx) {
 
 static int json_end_array(void *ctx) {
     LOG("end of array\n");
-    parsing_swallows = false;
+    if (!parsing_swallows && !parsing_focus) {
+        con_fix_percent(json_node);
+    }
+    if (parsing_swallows) {
+        parsing_swallows = false;
+    }
     if (parsing_focus) {
         /* Clear the list of focus mappings */
         struct focus_mapping *mapping;
-        TAILQ_FOREACH_REVERSE(mapping, &focus_mappings, focus_mappings_head, focus_mappings) {
+        TAILQ_FOREACH_REVERSE (mapping, &focus_mappings, focus_mappings_head, focus_mappings) {
             LOG("focus (reverse) %d\n", mapping->old_id);
             Con *con;
-            TAILQ_FOREACH(con, &(json_node->focus_head), focused) {
+            TAILQ_FOREACH (con, &(json_node->focus_head), focused) {
                 if (con->old_id != mapping->old_id)
                     continue;
                 LOG("got it! %p\n", con);
@@ -111,14 +133,10 @@ static int json_end_array(void *ctx) {
     return 1;
 }
 
-#if YAJL_MAJOR < 2
-static int json_key(void *ctx, const unsigned char *val, unsigned int len) {
-#else
 static int json_key(void *ctx, const unsigned char *val, size_t len) {
-#endif
     LOG("key: %.*s\n", (int)len, val);
     FREE(last_key);
-    last_key = scalloc((len+1) * sizeof(char));
+    last_key = scalloc((len + 1) * sizeof(char));
     memcpy(last_key, val, len);
     if (strcasecmp(last_key, "swallows") == 0)
         parsing_swallows = true;
@@ -138,25 +156,29 @@ static int json_key(void *ctx, const unsigned char *val, size_t len) {
     return 1;
 }
 
-#if YAJL_MAJOR >= 2
 static int json_string(void *ctx, const unsigned char *val, size_t len) {
-#else
-static int json_string(void *ctx, const unsigned char *val, unsigned int len) {
-#endif
     LOG("string: %.*s for key %s\n", (int)len, val, last_key);
     if (parsing_swallows) {
-        /* TODO: the other swallowing keys */
+        char *sval;
+        sasprintf(&sval, "%.*s", len, val);
         if (strcasecmp(last_key, "class") == 0) {
-            current_swallow->class = scalloc((len+1) * sizeof(char));
-            memcpy(current_swallow->class, val, len);
+            current_swallow->class = regex_new(sval);
+        } else if (strcasecmp(last_key, "instance") == 0) {
+            current_swallow->instance = regex_new(sval);
+        } else if (strcasecmp(last_key, "window_role") == 0) {
+            current_swallow->window_role = regex_new(sval);
+        } else if (strcasecmp(last_key, "title") == 0) {
+            current_swallow->title = regex_new(sval);
+        } else {
+            ELOG("swallow key %s unknown\n", last_key);
         }
-        LOG("unhandled yet: swallow\n");
+        free(sval);
     } else {
         if (strcasecmp(last_key, "name") == 0) {
-            json_node->name = scalloc((len+1) * sizeof(char));
+            json_node->name = scalloc((len + 1) * sizeof(char));
             memcpy(json_node->name, val, len);
         } else if (strcasecmp(last_key, "sticky_group") == 0) {
-            json_node->sticky_group = scalloc((len+1) * sizeof(char));
+            json_node->sticky_group = scalloc((len + 1) * sizeof(char));
             memcpy(json_node->sticky_group, val, len);
             LOG("sticky_group of this container is %s\n", json_node->sticky_group);
         } else if (strcasecmp(last_key, "orientation") == 0) {
@@ -173,7 +195,8 @@ static int json_string(void *ctx, const unsigned char *val, unsigned int len) {
                 json_node->last_split_layout = L_SPLITH;
             else if (strcasecmp(buf, "vertical") == 0)
                 json_node->last_split_layout = L_SPLITV;
-            else LOG("Unhandled orientation: %s\n", buf);
+            else
+                LOG("Unhandled orientation: %s\n", buf);
             free(buf);
         } else if (strcasecmp(last_key, "border") == 0) {
             char *buf = NULL;
@@ -187,7 +210,26 @@ static int json_string(void *ctx, const unsigned char *val, unsigned int len) {
                 json_node->border_style = BS_PIXEL;
             else if (strcasecmp(buf, "normal") == 0)
                 json_node->border_style = BS_NORMAL;
-            else LOG("Unhandled \"border\": %s\n", buf);
+            else
+                LOG("Unhandled \"border\": %s\n", buf);
+            free(buf);
+        } else if (strcasecmp(last_key, "type") == 0) {
+            char *buf = NULL;
+            sasprintf(&buf, "%.*s", (int)len, val);
+            if (strcasecmp(buf, "root") == 0)
+                json_node->type = CT_ROOT;
+            else if (strcasecmp(buf, "output") == 0)
+                json_node->type = CT_OUTPUT;
+            else if (strcasecmp(buf, "con") == 0)
+                json_node->type = CT_CON;
+            else if (strcasecmp(buf, "floating_con") == 0)
+                json_node->type = CT_FLOATING_CON;
+            else if (strcasecmp(buf, "workspace") == 0)
+                json_node->type = CT_WORKSPACE;
+            else if (strcasecmp(buf, "dockarea") == 0)
+                json_node->type = CT_DOCKAREA;
+            else
+                LOG("Unhandled \"type\": %s\n", buf);
             free(buf);
         } else if (strcasecmp(last_key, "layout") == 0) {
             char *buf = NULL;
@@ -207,7 +249,8 @@ static int json_string(void *ctx, const unsigned char *val, unsigned int len) {
                 json_node->layout = L_SPLITH;
             else if (strcasecmp(buf, "splitv") == 0)
                 json_node->layout = L_SPLITV;
-            else LOG("Unhandled \"layout\": %s\n", buf);
+            else
+                LOG("Unhandled \"layout\": %s\n", buf);
             free(buf);
         } else if (strcasecmp(last_key, "workspace_layout") == 0) {
             char *buf = NULL;
@@ -218,7 +261,8 @@ static int json_string(void *ctx, const unsigned char *val, unsigned int len) {
                 json_node->workspace_layout = L_STACKED;
             else if (strcasecmp(buf, "tabbed") == 0)
                 json_node->workspace_layout = L_TABBED;
-            else LOG("Unhandled \"workspace_layout\": %s\n", buf);
+            else
+                LOG("Unhandled \"workspace_layout\": %s\n", buf);
             free(buf);
         } else if (strcasecmp(last_key, "last_split_layout") == 0) {
             char *buf = NULL;
@@ -227,7 +271,8 @@ static int json_string(void *ctx, const unsigned char *val, unsigned int len) {
                 json_node->last_split_layout = L_SPLITH;
             else if (strcasecmp(buf, "splitv") == 0)
                 json_node->last_split_layout = L_SPLITV;
-            else LOG("Unhandled \"last_splitlayout\": %s\n", buf);
+            else
+                LOG("Unhandled \"last_splitlayout\": %s\n", buf);
             free(buf);
         } else if (strcasecmp(last_key, "mark") == 0) {
             char *buf = NULL;
@@ -260,13 +305,9 @@ static int json_string(void *ctx, const unsigned char *val, unsigned int len) {
     return 1;
 }
 
-#if YAJL_MAJOR >= 2
 static int json_int(void *ctx, long long val) {
     LOG("int %lld for key %s\n", val, last_key);
-#else
-static int json_int(void *ctx, long val) {
-    LOG("int %ld for key %s\n", val, last_key);
-#endif
+    /* For backwards compatibility with i3 < 4.8 */
     if (strcasecmp(last_key, "type") == 0)
         json_node->type = val;
 
@@ -297,7 +338,8 @@ static int json_int(void *ctx, long val) {
             r = &(json_node->rect);
         else if (parsing_window_rect)
             r = &(json_node->window_rect);
-        else r = &(json_node->geometry);
+        else
+            r = &(json_node->geometry);
         if (strcasecmp(last_key, "x") == 0)
             r->x = val;
         else if (strcasecmp(last_key, "y") == 0)
@@ -306,9 +348,10 @@ static int json_int(void *ctx, long val) {
             r->width = val;
         else if (strcasecmp(last_key, "height") == 0)
             r->height = val;
-        else printf("WARNING: unknown key %s in rect\n", last_key);
-        printf("rect now: (%d, %d, %d, %d)\n",
-                r->x, r->y, r->width, r->height);
+        else
+            ELOG("WARNING: unknown key %s in rect\n", last_key);
+        DLOG("rect now: (%d, %d, %d, %d)\n",
+             r->x, r->y, r->width, r->height);
     }
     if (parsing_swallows) {
         if (strcasecmp(last_key, "id") == 0) {
@@ -347,8 +390,7 @@ static int json_double(void *ctx, double val) {
     return 1;
 }
 
-void tree_append_json(const char *filename) {
-    /* TODO: percent of other windows are not correctly fixed at the moment */
+void tree_append_json(Con *con, const char *filename, char **errormsg) {
     FILE *f;
     if ((f = fopen(filename, "r")) == NULL) {
         LOG("Cannot open file \"%s\"\n", filename);
@@ -370,44 +412,47 @@ void tree_append_json(const char *filename) {
     LOG("read %d bytes\n", n);
     yajl_gen g;
     yajl_handle hand;
-    yajl_callbacks callbacks;
-    memset(&callbacks, '\0', sizeof(yajl_callbacks));
-    callbacks.yajl_start_map = json_start_map;
-    callbacks.yajl_end_map = json_end_map;
-    callbacks.yajl_end_array = json_end_array;
-    callbacks.yajl_string = json_string;
-    callbacks.yajl_map_key = json_key;
-    callbacks.yajl_integer = json_int;
-    callbacks.yajl_double = json_double;
-    callbacks.yajl_boolean = json_bool;
-#if YAJL_MAJOR >= 2
+    static yajl_callbacks callbacks = {
+        .yajl_boolean = json_bool,
+        .yajl_integer = json_int,
+        .yajl_double = json_double,
+        .yajl_string = json_string,
+        .yajl_start_map = json_start_map,
+        .yajl_map_key = json_key,
+        .yajl_end_map = json_end_map,
+        .yajl_end_array = json_end_array,
+    };
     g = yajl_gen_alloc(NULL);
-    hand = yajl_alloc(&callbacks, NULL, (void*)g);
-#else
-    g = yajl_gen_alloc(NULL, NULL);
-    hand = yajl_alloc(&callbacks, NULL, NULL, (void*)g);
-#endif
+    hand = yajl_alloc(&callbacks, NULL, (void *)g);
+    /* Allowing comments allows for more user-friendly layout files. */
+    yajl_config(hand, yajl_allow_comments, true);
+    /* Allow multiple values, i.e. multiple nodes to attach */
+    yajl_config(hand, yajl_allow_multiple_values, true);
     yajl_status stat;
-    json_node = focused;
+    json_node = con;
     to_focus = NULL;
+    parsing_swallows = false;
     parsing_rect = false;
     parsing_window_rect = false;
     parsing_geometry = false;
+    parsing_focus = false;
     setlocale(LC_NUMERIC, "C");
-    stat = yajl_parse(hand, (const unsigned char*)buf, n);
-    if (stat != yajl_status_ok)
-    {
-        unsigned char * str = yajl_get_error(hand, 1, (const unsigned char*)buf, n);
-        fprintf(stderr, "%s\n", (const char *) str);
+    stat = yajl_parse(hand, (const unsigned char *)buf, n);
+    if (stat != yajl_status_ok) {
+        unsigned char *str = yajl_get_error(hand, 1, (const unsigned char *)buf, n);
+        ELOG("JSON parsing error: %s\n", str);
+        if (errormsg != NULL)
+            *errormsg = sstrdup((const char *)str);
         yajl_free_error(hand, str);
     }
 
+    /* In case not all containers were restored, we need to fix the
+     * percentages, otherwise i3 will crash immediately when rendering the
+     * next time. */
+    con_fix_percent(con);
+
     setlocale(LC_NUMERIC, "");
-#if YAJL_MAJOR >= 2
     yajl_complete_parse(hand);
-#else
-    yajl_parse_complete(hand);
-#endif
 
     fclose(f);
     if (to_focus)
