@@ -62,6 +62,14 @@ sub activate_i3 {
             mkdir $ENV{XDG_RUNTIME_DIR};
         }
         $ENV{DISPLAY} = $args{display};
+        $ENV{PATH} = join(':',
+            '../i3-nagbar',
+            '../i3-msg',
+            '../i3-config-wizard',
+            '../i3bar',
+            '..',
+            $ENV{PATH}
+        );
 
         # We are about to exec, but we did not modify $^F to include $socket
         # when creating the socket (because the file descriptor could have a
@@ -88,13 +96,8 @@ sub activate_i3 {
         # the interactive signalhandler to make it crash immediately instead.
         # Also disable logging to SHM since we redirect the logs anyways.
         # Force Xinerama because we use Xdmx for multi-monitor tests.
-        my $i3cmd = q|i3 --shmlog-size=0 --disable-signalhandler --force-xinerama|;
-        if (!$args{validate_config}) {
-            # We only set logging if i3 is actually started, but not if we only
-            # validate the config file. This is to keep logging to a minimum as
-            # such a test will likely want to inspect the log file.
-            $i3cmd .= q| -V -d all|;
-        }
+        my $i3cmd = abs_path("../i3") . q| -V -d all --disable-signalhandler| .
+                                        q| --shmlog-size=0 --force-xinerama|;
 
         # For convenience:
         my $outdir = $args{outdir};
@@ -104,13 +107,9 @@ sub activate_i3 {
             $i3cmd .= ' -L ' . abs_path('restart-state.golden');
         }
 
-        if ($args{validate_config}) {
-            $i3cmd .= ' -C';
-        }
-
         if ($args{valgrind}) {
             $i3cmd =
-                qq|valgrind --log-file="$outdir/valgrind-for-$test.log" | .
+                qq|valgrind -v --log-file="$outdir/valgrind-for-$test.log" | .
                 qq|--suppressions="./valgrind.supp" | .
                 qq|--leak-check=full --track-origins=yes --num-callers=20 | .
                 qq|--tool=memcheck -- $i3cmd|;
@@ -150,11 +149,6 @@ sub activate_i3 {
     # descriptor on the listening socket.
     $socket->close;
 
-    if ($args{validate_config}) {
-        $args{cv}->send(1);
-        return $pid;
-    }
-
     # We now connect (will succeed immediately) and send a request afterwards.
     # As soon as the reply is there, i3 is considered ready.
     my $cl = IO::Socket::UNIX->new(Peer => $args{unix_socket_path});
